@@ -2,6 +2,7 @@
 from typing import List, Union, Tuple
 from scipy.stats import beta, chi
 from matplotlib import pyplot as plt
+from scipy.stats import norm
 import numpy as np
 import scipy.stats
 import scipy.special as sc
@@ -32,6 +33,7 @@ class ExperimentGuideSets:
         self.positive_control_guides = positive_control_guides
         self.observation_guides = observation_guides
 
+# TODO: Change to dataclass
 class CrisprShrinkageResult:
     '''Represents the final object passed back with the adjusted scores'''
 
@@ -39,6 +41,8 @@ class CrisprShrinkageResult:
             adjusted_negative_control_guides: List[Guide],
             adjusted_observation_guides: List[Guide],
             adjusted_positive_control_guides: List[Guide],
+            negative_control_guide_sample_population_total_normalized_counts_reps: List[float],
+            negative_control_guide_control_population_total_normalized_counts_reps: List[float],
             raw_negative_control_guides: List[Guide],
             raw_positive_control_guides: List[Guide],
             raw_observation_guides: List[Guide],
@@ -62,6 +66,8 @@ class CrisprShrinkageResult:
         self.adjusted_negative_control_guides=adjusted_negative_control_guides
         self.adjusted_observation_guides=adjusted_observation_guides
         self.adjusted_positive_control_guides=adjusted_positive_control_guides
+        self.negative_control_guide_sample_population_total_normalized_counts_reps = negative_control_guide_sample_population_total_normalized_counts_reps
+        self.negative_control_guide_control_population_total_normalized_counts_reps = negative_control_guide_control_population_total_normalized_counts_reps
         self.shrinkage_prior_strength=shrinkage_prior_strength
         self.neighborhood_imputation_prior_strength=neighborhood_imputation_prior_strength
         self.neighborhood_imputation_likelihood_strength=neighborhood_imputation_likelihood_strength
@@ -208,7 +214,26 @@ class StatisticalHelperMethods:
             LFC_posterior_mean_per_guide_M_BP_pval = 1-chi.cdf(LFC_posterior_mean_per_guide_M_BP_statistic, 1) # TODO: Double check if the degree of freedom is correct
 
             return LFC_posterior_mean_per_guide_M_BP_statistic
+    
+    @staticmethod
+    def calculate_norm_prob_0(loc, scale):
+        return norm.pdf(0, loc, scale)
 
+    @staticmethod
+    def calculate_norm_lt_0(loc, scale):
+        return cdf(0, loc, scale)
+
+    @staticmethod
+    def calculate_norm_lt_0_mc(monte_carlo_samples: List[float]):
+        return sum(monte_carlo_samples<0)/len(monte_carlo_samples)
+    
+    @staticmethod
+    def calculate_norm_gt_0(loc, scale):
+        return 1-StatisticalHelperMethods.calculate_norm_lt_0(loc, scale)
+
+    @staticmethod
+    def calculate_norm_gt_0_mc(monte_carlo_samples: List[float]):
+        return sum(monte_carlo_samples>0)/len(monte_carlo_samples)
 
 #-------------------------Helper and Inference functions
 def determine_guide_fit(guide: Guide, contains_position: Union[bool, None]):
@@ -377,9 +402,9 @@ def singleton_imputation_objective_function_of_all_guides(rep_i: int,
                                     negative_control_guide_sample_population_total_normalized_counts_reps: List[float],
                                     negative_control_guide_control_population_total_normalized_counts_reps: List[float],
                                     params: float):
-    KL_guide_imputation_score_total_negative, negative_total_n = SingletonImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.negative_control_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
-    KL_guide_imputation_score_total_positive, positive_total_n = SingletonImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.positive_control_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
-    KL_guide_imputation_score_total_observation, observation_total_n = SingletonImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.observation_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
+    KL_guide_imputation_score_total_negative, negative_total_n = singleton_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.negative_control_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
+    KL_guide_imputation_score_total_positive, positive_total_n = singleton_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.positive_control_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
+    KL_guide_imputation_score_total_observation, observation_total_n = singleton_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.observation_guides, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
     
     total_n = (negative_total_n+positive_total_n+observation_total_n)
     
@@ -398,7 +423,7 @@ def singleton_imputation_weighted_objective_function_of_all_guides(rep_i: int,
                                                         negative_control_guide_control_population_total_normalized_counts_reps: List[float],
                                                         KL_guide_set_weights: Union[List[float], None],
                                                         params: float):
-            KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg = SingletonImputationOptimizer.singleton_imputation_objective_function_of_all_guides(rep_i, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
+            KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg = singleton_imputation_objective_function_of_all_guides(rep_i, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, params)
 
             if KL_guide_set_weights is None:
                 return KL_guide_imputation_score_total_combined_avg
@@ -445,7 +470,7 @@ class SingletonImputationOptimizer:
             tolerance = optimizer_upper_bound / 1000
             print("Singleton optimization upper bound: {}".format(optimizer_upper_bound))
             print("Tolerance: {}".format(tolerance))
-            res = scipy.optimize.differential_evolution(weighted_objective_function_of_all_guides_p, bounds=[(0.000001, optimizer_upper_bound)], callback=store_values, maxiter= 10000, tol=0.005, workers=cores) 
+            res = scipy.optimize.differential_evolution(weighted_objective_function_of_all_guides_p, bounds=[(0.000001, optimizer_upper_bound)], callback=store_values, maxiter= 10000, tol=0.05, workers=cores) 
 
             plt.scatter(param_vals, loss_vals)
             plt.xlabel("Prior Strength")
@@ -527,11 +552,11 @@ def neighborhood_imputation_objective_function_of_all_guides(rep_i: int,
     
     # Iterate through each guide to test prior with tested weight
 
-    KL_guide_imputation_score_total_negative, negative_sampled_n, negative_total_n = NeighborhoodImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.negative_control_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
+    KL_guide_imputation_score_total_negative, negative_sampled_n, negative_total_n = neighborhood_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.negative_control_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
 
-    KL_guide_imputation_score_total_positive, positive_sampled_n, positive_total_n = NeighborhoodImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.positive_control_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
+    KL_guide_imputation_score_total_positive, positive_sampled_n, positive_total_n = neighborhood_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.positive_control_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
 
-    KL_guide_imputation_score_total_observation, observation_sampled_n, observation_total_n = NeighborhoodImputationOptimizer.objective_function_of_each_guide_set(rep_i, experiment_guide_sets.observation_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
+    KL_guide_imputation_score_total_observation, observation_sampled_n, observation_total_n = neighborhood_imputation_objective_function_of_each_guide_set(rep_i, experiment_guide_sets.observation_guides, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
     
     total_n = (negative_total_n+positive_total_n+observation_total_n)
     
@@ -547,12 +572,13 @@ def neighborhood_imputation_objective_function_of_all_guides(rep_i: int,
     return KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg
 
 def neighborhood_imputation_weighted_objective_function_of_all_guides(rep_i: int, 
+                                        neighborhood_optimization_guide_sample_size: int,
                                         experiment_guide_sets: ExperimentGuideSets,
                                         negative_control_guide_sample_population_total_normalized_counts_reps: List[float],
                                         negative_control_guide_control_population_total_normalized_counts_reps: List[float],
                                         neighborhood_bandwidth: float,
                                         deviation_weights: List[float],
-                                        guide_set_weights: Union[List[float], None], 
+                                        KL_guide_set_weights: Union[List[float], None], 
                                         params: Tuple[float, float]):
     KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg = NeighborhoodImputationOptimizer.neighborhood_imputation_objective_function_of_all_guides(rep_i, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, params)
     
@@ -583,7 +609,7 @@ class NeighborhoodImputationOptimizer:
         neighborhood_imputation_likelihood_strength_selected: List[float] = []
 
         for rep_i in replicate_indices:
-            weighted_objective_function_of_all_guides_p = functools.partial(neighborhood_imputation_weighted_objective_function_of_all_guides, rep_i, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, KL_guide_set_weights)
+            weighted_objective_function_of_all_guides_p = functools.partial(neighborhood_imputation_weighted_objective_function_of_all_guides, rep_i, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, KL_guide_set_weights)
 
 
             param_vals=[]
@@ -602,7 +628,8 @@ class NeighborhoodImputationOptimizer:
             optimizer_upper_bound = np.min([optimizer_bounds_a, optimizer_bounds_b])
             tolerance = optimizer_upper_bound / 1000
             print("Neighborhood optimization upper bound: {}".format(optimizer_upper_bound))
-            res = scipy.optimize.differential_evolution(weighted_objective_function_of_all_guides_p, bounds=[(0.000001, optimizer_upper_bound),(0.000001, optimizer_upper_bound)], callback=store_values, tol=0.005, workers=cores) 
+
+            res = scipy.optimize.differential_evolution(weighted_objective_function_of_all_guides_p, bounds=[(0.000001, optimizer_upper_bound),(0.000001, optimizer_upper_bound)], callback=store_values, tol=0.05, workers=cores) 
 
             X=[param[0] for param in param_vals]
             Y=[param[1] for param in param_vals]
@@ -622,7 +649,7 @@ class NeighborhoodImputationOptimizer:
             if res.success is True:
                 neighborhood_imputation_prior_strength, neighborhood_imputation_likelihood_strength = res.x
                 
-                KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg = NeighborhoodImputationOptimizer.objective_function_of_all_guides(rep_i, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, res.x)
+                KL_guide_imputation_score_total_combined_avg, KL_guide_imputation_score_total_negative_avg, KL_guide_imputation_score_total_positive_avg, KL_guide_imputation_score_total_observation_avg = neighborhood_imputation_objective_function_of_all_guides(rep_i, neighborhood_optimization_guide_sample_size, experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, neighborhood_bandwidth, deviation_weights, res.x)
 
                 print("KL Negative Set Average: {}".format(KL_guide_imputation_score_total_negative_avg))
                 print("KL Positive Set Average: {}".format(KL_guide_imputation_score_total_positive_avg))
@@ -642,7 +669,6 @@ class NeighborhoodImputationOptimizer:
 
 
 
-@staticmethod
 def shrinkage_optimizer_weighted_objective_function_of_all_guides(rep_i: int, 
                                         spatial_guides_for_fit: List[Guide],
                                         singleton_guides_for_fit: List[Guide], 
@@ -837,6 +863,7 @@ def perform_adjustment(
     KL_guide_set_weights: Union[List[float], None] = None, 
     shrinkage_prior_strength: Union[List[float], None] = None,
     monte_carlo_trials: int = 1000,
+    neighborhood_optimization_guide_sample_size:int = 50,
     posterior_estimator: str = "mean",
     random_seed: Union[int, None] = None,
     cores=1
@@ -854,11 +881,11 @@ def perform_adjustment(
 
     replicate_indices = np.asarray(range(num_replicates))
 
-    for guide in negative_control_guides:
+    for guide in raw_negative_control_guides:
         assert num_replicates == len(guide.sample_population_raw_count_reps) == len(guide.control_population_raw_count_reps), "Guide {} number of counts does not equal replicates"
-    for guide in observation_guides:
+    for guide in raw_positive_control_guides:
         assert num_replicates == len(guide.sample_population_raw_count_reps) == len(guide.control_population_raw_count_reps), "Guide {} number of counts does not equal replicates"
-    for guide in observation_guides:
+    for guide in raw_observation_guides:
         assert num_replicates == len(guide.sample_population_raw_count_reps) == len(guide.control_population_raw_count_reps), "Guide {} number of counts does not equal replicates"
 
 
@@ -876,9 +903,9 @@ def perform_adjustment(
             guide.sample_population_normalized_count_reps = (guide.sample_population_raw_count_reps/sample_population_scaling_factors) + 1
             guide.control_population_normalized_count_reps = (guide.control_population_raw_count_reps/control_population_scaling_factors) + 1
 
-    normalize_guide_counts(negative_control_guides, sample_population_scaling_factors, control_population_scaling_factors)
-    normalize_guide_counts(positive_control_guides, sample_population_scaling_factors, control_population_scaling_factors)
-    normalize_guide_counts(observation_guides, sample_population_scaling_factors, control_population_scaling_factors)
+    normalize_guide_counts(raw_negative_control_guides, sample_population_scaling_factors, control_population_scaling_factors)
+    normalize_guide_counts(raw_positive_control_guides, sample_population_scaling_factors, control_population_scaling_factors)
+    normalize_guide_counts(raw_observation_guides, sample_population_scaling_factors, control_population_scaling_factors)
     
     
     if neighborhood_imputation_prior_strength is not None:
@@ -904,15 +931,15 @@ def perform_adjustment(
         KL_guide_set_weights = np.asarray(KL_guide_set_weights)
 
     # Create all guides set used for informing neighborhood prior, performing final shrinkage, and visualization    
-    experiment_guide_sets: ExperimentGuideSets = ExperimentGuideSets(negative_control_guides, positive_control_guides, observation_guides)
+    experiment_guide_sets: ExperimentGuideSets = ExperimentGuideSets(raw_negative_control_guides, raw_positive_control_guides, raw_observation_guides)
 
     # Get total normalized counts of negative controls in both populations to be used as initial prior
     negative_control_guide_sample_population_total_normalized_counts_reps: List[int] = np.repeat(0., num_replicates)
     negative_control_guide_control_population_total_normalized_counts_reps: List[int] = np.repeat(0., num_replicates)
-    negative_control_guide: Guide
-    for negative_control_guide in negative_control_guides:
-            negative_control_guide_sample_population_total_normalized_counts_reps = negative_control_guide_sample_population_total_normalized_counts_reps + negative_control_guide.sample_population_normalized_count_reps
-            negative_control_guide_control_population_total_normalized_counts_reps = negative_control_guide_control_population_total_normalized_counts_reps + negative_control_guide.control_population_normalized_count_reps
+    raw_negative_control_guide: Guide
+    for raw_negative_control_guide in raw_negative_control_guides:
+            negative_control_guide_sample_population_total_normalized_counts_reps = negative_control_guide_sample_population_total_normalized_counts_reps + raw_negative_control_guide.sample_population_normalized_count_reps
+            negative_control_guide_control_population_total_normalized_counts_reps = negative_control_guide_control_population_total_normalized_counts_reps + raw_negative_control_guide.control_population_normalized_count_reps
 
 
     neighborhood_experiment_guide_sets: ExperimentGuideSets = None
@@ -936,7 +963,7 @@ def perform_adjustment(
     if enable_neighborhood_prior:
         if (neighborhood_imputation_prior_strength is None) or (neighborhood_imputation_likelihood_strength is None):
             print("Optimizing neighborhood imputation weights")
-            neighborhood_imputation_prior_strength,neighborhood_imputation_likelihood_strength = NeighborhoodImputationOptimizer.optimize_neighborhood_imputation_prior_strength(neighborhood_experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, replicate_indices, neighborhood_bandwidth, deviation_weights, KL_guide_set_weights, cores)
+            neighborhood_imputation_prior_strength,neighborhood_imputation_likelihood_strength = NeighborhoodImputationOptimizer.optimize_neighborhood_imputation_prior_strength(neighborhood_experiment_guide_sets, negative_control_guide_sample_population_total_normalized_counts_reps, negative_control_guide_control_population_total_normalized_counts_reps, replicate_indices, neighborhood_bandwidth, deviation_weights, KL_guide_set_weights, cores, neighborhood_optimization_guide_sample_size)
             print("Selected neighborhood imputation weights: prior={}, likelihood={}".format(neighborhood_imputation_prior_strength, neighborhood_imputation_likelihood_strength))
         
     if singleton_imputation_prior_strength is None:
@@ -979,9 +1006,11 @@ def perform_adjustment(
         guide_count_posterior_LFC_samples_normalized_average = np.mean(guide_count_posterior_LFC_samples_normalized_list, axis=0)
 
         LFC_estimate_combined = None
+        LFC_estimate_combined_mean = np.mean(guide_count_posterior_LFC_samples_normalized_average)
+        LFC_estimate_per_replicate_mean = np.mean(guide_count_posterior_LFC_samples_normalized_list, axis=1)
         if posterior_estimator.upper() == "MEAN":
-            LFC_estimate_combined = np.mean(guide_count_posterior_LFC_samples_normalized_average)
-            LFC_estimate_per_replicate = np.mean(guide_count_posterior_LFC_samples_normalized_list, axis=1)
+            LFC_estimate_combined = LFC_estimate_combined_mean
+            LFC_estimate_per_replicate = LFC_estimate_per_replicate_mean
         elif posterior_estimator.upper() == "MODE":
             LFC_estimate_combined = StatisticalHelperMethods.calculate_map(guide_count_posterior_LFC_samples_normalized_average)
             LFC_estimate_per_replicate =  np.asarray([StatisticalHelperMethods.calculate_map(guide_count_posterior_LFC_samples_normalized) for guide_count_posterior_LFC_samples_normalized in guide_count_posterior_LFC_samples_normalized_list])
@@ -993,6 +1022,16 @@ def perform_adjustment(
         LFC_estimate_per_replicate_std = np.asarray([np.std(guide_count_posterior_LFC_samples_normalized_rep) for guide_count_posterior_LFC_samples_normalized_rep in guide_count_posterior_LFC_samples_normalized_list])
 
 
+        LFC_estimate_combined_prob0 = StatisticalHelperMethods.calculate_norm_prob_0(LFC_estimate_combined_mean, LFC_estimate_combined_std)
+        LFC_estimate_per_replicate_prob0 = [StatisticalHelperMethods.calculate_norm_prob_0(LFC_estimate_per_replicate_mean[rep_i], LFC_estimate_per_replicate_std[rep_i]) for rep_i in replicate_indices]
+
+        LFC_estimate_combined_prob_lt0 = StatisticalHelperMethods.calculate_norm_lt_0_mc(guide_count_posterior_LFC_samples_normalized_average)
+        LFC_estimate_combined_prob_gt0 = StatisticalHelperMethods.calculate_norm_gt_0_mc(guide_count_posterior_LFC_samples_normalized_average)
+        
+        LFC_estimate_per_replicate_lt0 = [StatisticalHelperMethods.calculate_norm_lt_0_mc(guide_count_posterior_LFC_samples_normalized_rep_i) for guide_count_posterior_LFC_samples_normalized_rep_i in guide_count_posterior_LFC_samples_normalized_list]
+        LFC_estimate_per_replicate_gt0 = [StatisticalHelperMethods.calculate_norm_gt_0_mc(guide_count_posterior_LFC_samples_normalized_rep_i) for guide_count_posterior_LFC_samples_normalized_rep_i in guide_count_posterior_LFC_samples_normalized_list]
+
+
         each_guide.LFC_estimate_combined = LFC_estimate_combined
         each_guide.LFC_estimate_per_replicate = LFC_estimate_per_replicate
 
@@ -1001,7 +1040,18 @@ def perform_adjustment(
 
         each_guide.LFC_estimate_per_replicate_CI = LFC_estimate_per_replicate_CI 
         each_guide.LFC_estimate_per_replicate_std = LFC_estimate_per_replicate_std
-        
+
+        each_guide.LFC_estimate_combined_prob0 = LFC_estimate_combined_prob0
+        each_guide.LFC_estimate_combined_prob_lt0 = LFC_estimate_combined_prob_lt0
+        each_guide.LFC_estimate_combined_prob_gt0 = LFC_estimate_combined_prob_gt0
+
+        each_guide.LFC_estimate_per_replicate_prob0 = LFC_estimate_per_replicate_prob0
+        each_guide.LFC_estimate_per_replicate_lt0 = LFC_estimate_per_replicate_lt0
+        each_guide.LFC_estimate_per_replicate_gt0 = LFC_estimate_per_replicate_gt0
+
+        each_guide.guide_count_posterior_LFC_samples_normalized_list = guide_count_posterior_LFC_samples_normalized_list
+        each_guide.guide_count_posterior_LFC_samples_normalized_average = guide_count_posterior_LFC_samples_normalized_average
+
         return each_guide
 
     # Perform final model inference:
@@ -1063,20 +1113,22 @@ def perform_adjustment(
         singletons_experiment_guide_sets.positive_control_guides = inference_singleton_guide_set(singletons_experiment_guide_sets.positive_control_guides)
 
 
-        negative_control_guides = np.concatenate([neighborhood_experiment_guide_sets.negative_control_guides, singletons_experiment_guide_sets.negative_control_guides])
-        observation_guides = np.concatenate([neighborhood_experiment_guide_sets.observation_guides, singletons_experiment_guide_sets.observation_guides])
-        positive_control_guides = np.concatenate([neighborhood_experiment_guide_sets.positive_control_guides, singletons_experiment_guide_sets.positive_control_guides])
+        adjusted_negative_control_guides = np.concatenate([neighborhood_experiment_guide_sets.negative_control_guides, singletons_experiment_guide_sets.negative_control_guides])
+        adjusted_observation_guides = np.concatenate([neighborhood_experiment_guide_sets.observation_guides, singletons_experiment_guide_sets.observation_guides])
+        adjusted_positive_control_guides = np.concatenate([neighborhood_experiment_guide_sets.positive_control_guides, singletons_experiment_guide_sets.positive_control_guides])
 
     else:
-        negative_control_guides = inference_singleton_guide_set(experiment_guide_sets.negative_control_guides)
-        observation_guides = inference_singleton_guide_set(experiment_guide_sets.observation_guides)
-        positive_control_guides = inference_singleton_guide_set(experiment_guide_sets.positive_control_guides)
+        adjusted_negative_control_guides = inference_singleton_guide_set(experiment_guide_sets.negative_control_guides)
+        adjusted_observation_guides = inference_singleton_guide_set(experiment_guide_sets.observation_guides)
+        adjusted_positive_control_guides = inference_singleton_guide_set(experiment_guide_sets.positive_control_guides)
 
 
     return CrisprShrinkageResult(
-        adjusted_negative_control_guides=negative_control_guides,
-        adjusted_observation_guides=observation_guides,
-        adjusted_positive_control_guides=positive_control_guides,
+        adjusted_negative_control_guides=adjusted_negative_control_guides,
+        adjusted_observation_guides=adjusted_observation_guides,
+        adjusted_positive_control_guides=adjusted_positive_control_guides,
+        negative_control_guide_sample_population_total_normalized_counts_reps=negative_control_guide_sample_population_total_normalized_counts_reps,
+        negative_control_guide_control_population_total_normalized_counts_reps=negative_control_guide_control_population_total_normalized_counts_reps,
         shrinkage_prior_strength=shrinkage_prior_strength,
         neighborhood_imputation_prior_strength=neighborhood_imputation_prior_strength,
         neighborhood_imputation_likelihood_strength=neighborhood_imputation_likelihood_strength,
@@ -1116,7 +1168,7 @@ if __name__ == "__main__":
 
     reps = 3
     max_dup_factor = 30
-    max_guide_molecule_factor = 50
+    max_guide_molecule_factor = 20
 
 
     get_positive_proportion = lambda pos_prop: ((1-pos_prop)*target_null_proportion) + (pos_prop)*target_positive_population
@@ -1124,6 +1176,8 @@ if __name__ == "__main__":
     pop1_dup_factor_list = np.asarray([np.round(uniform.rvs(1, max_dup_factor)) for _ in range(reps)])
     pop2_dup_factor_list = np.asarray([np.round(uniform.rvs(1, max_dup_factor)) for _ in range(reps)])
 
+    # This defines the function to generate the counts
+
     #expon.rvs(loc=1, scale=1000, size=num_guides)
     #uniform.rvs(2, 200, size=num_guides)
     def get_counts(num_guides, proportion):
@@ -1140,7 +1194,8 @@ if __name__ == "__main__":
         
         return np.asarray(pop1_list_reps), np.asarray(pop2_list_reps)
 
-    
+    # This prepares the groun_truth regions
+
     get_kernel_values = lambda position, b,xrange : np.asarray([StatisticalHelperMethods.gaussian_kernel(i, position, b) for i in range(position-xrange, position+xrange+1)])
 
     normalize_kernel_values = lambda kernel_values: (kernel_values-kernel_values.min())/(kernel_values.max() - kernel_values.min())
@@ -1153,7 +1208,6 @@ if __name__ == "__main__":
 
     tiling_length = 300
 
-
     positive_regions = [(50,5,kernel_values_50), (100,3,kernel_values_100), (150,10,kernel_values_150), (200,15,kernel_values_200)]
     observation_guides = []
     for position in range(tiling_length):
@@ -1167,7 +1221,7 @@ if __name__ == "__main__":
 
         pop1_raw_count_reps = counts[0].transpose()[0]
         pop2_raw_count_reps = counts[1].transpose()[0]
-        guide = Guide(identifier="observation_{}".format(position), position=position, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps)
+        guide = Guide(identifier="observation_{}".format(position), position=position, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps, is_explanatory=True)
 
         observation_guides.append(guide)
 
@@ -1176,45 +1230,42 @@ if __name__ == "__main__":
         counts = get_counts(1, null_proportion)
         pop1_raw_count_reps = counts[0].transpose()[0]
         pop2_raw_count_reps = counts[1].transpose()[0]
-        guide = Guide(identifier="negative_{}".format(i), position=None, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps)
-
+        guide = Guide(identifier="negative_{}".format(i), position=None, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps, is_explanatory=False)
         negative_guides.append(guide)
 
     negative_guides = np.asarray(negative_guides)
 
-
     positive_guides = []
     for i in range(num_pos_guides):
         counts = get_counts(1, positive_proportion)
         pop1_raw_count_reps = counts[0].transpose()[0] + 1
         pop2_raw_count_reps = counts[1].transpose()[0] + 1
-        guide = Guide(identifier="positive_{}".format(i), position=None, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps)
-
+        guide = Guide(identifier="positive_{}".format(i), position=None, sample_population_raw_count_reps= pop1_raw_count_reps, control_population_raw_count_reps=pop2_raw_count_reps, is_explanatory=False)
         positive_guides.append(guide)
 
     positive_guides = np.asarray(positive_guides)
 
-
+    # LEFTOFF - just modifed result to return each guide set separately. So should be able to plot by position and verify that the shrinkage and all works well. Very interested to see if the negatiev controls are over shrunk. and positive controls. since they dont have position.
     results = perform_adjustment(
         negative_control_guides = negative_guides,
         positive_control_guides = positive_guides,
         observation_guides = observation_guides,
-        num_replicates = reps,
+        num_replicates = 3,
         include_observational_guides_in_fit = False,
         include_positive_control_guides_in_fit = False,
         sample_population_scaling_factors = pop1_dup_factor_list,
         control_population_scaling_factors = pop2_dup_factor_list,
         monte_carlo_trials = 1000,
+        neighborhood_optimization_guide_sample_size = 500,
         enable_neighborhood_prior =  True,
         neighborhood_bandwidth = 7,
-        neighborhood_imputation_prior_strength = [0.0001549,  0.00051831, 0.00070273],
-        neighborhood_imputation_likelihood_strength = [0.39062681, 0.3020106, 0.17357199], 
-        singleton_imputation_prior_strength = [0.00142066, 0.0013584,  0.00141909],
-        deviation_weights = [1,1,1],
+        neighborhood_imputation_prior_strength = [0.00123287, 0.00126275, 0.00129235],
+        neighborhood_imputation_likelihood_strength = [0.05219207, 0.05339028, 0.05260389],
+        singleton_imputation_prior_strength = None,
+        deviation_weights = np.asarray([10,10,10]),
         KL_guide_set_weights = None,
-        shrinkage_prior_strength = [1.50897246, 1.08297809, 2.45095095],
+        shrinkage_prior_strength = None,
         posterior_estimator = "mean",
-        random_seed = 234
-        )
-
-    print(results)
\ No newline at end of file
+        random_seed = 234,
+        cores=10
+    )
\ No newline at end of file
